# Command Cheat Sheet
This cheat sheet provides quick access to essential Linux, module, and SLURM commands commonly used on Berzelius. It’s designed as a reference for both new and experienced users to streamline development, job submission, monitoring, and resource management.

## Apptainer Commands

### apptainer build
- Build images	
```
apptainer build –fakeroot apptainer_image.sif apptainer_image.def
```

### apptainer exec
- Execute commands	
```
apptainer exec –nv -B /local_data_dir:/data apptainer_image.sif bash -c “python some_script.py”
```


### apptainer pull
- Download images	
```
apptainer pull pytorch_2.0.1.sif docker://pytorch/pytorch:2.0.1-cuda11.7-cudnn8-runtime
```

### apptainer shell
- Initialize a shell	
```
apptainer shell –nv apptainer_image.sif
```

## Git Commands

### git add
- Stage changes in your Git repository
```bash
git add my_script.py
```

### git clone
- Download a full copy of a Git repository
```bash
git clone <repository-url>
```

### git commit
- Commit changes
```bash
git commit -m "Message"
```

### git push
- Upload your local commits
```bash
git push
```

### git status
- Check what’s staged (in green) and what’s still unstaged (in red)
```bash
git status
```


## Linux Commands

### ncdu
- Show disk usage
```bash
ncdu /proj/nsc_testing/xuan
```
### nvidia-smi

- Show the current status of NVIDIA GPUs 
```bash
nvidia-smi
```

### rsync
- Copy local directory to another local location
```bash
rsync -av my_data/ backup/my_data/
```
- Copy local directory to a remote server over SSH
```bash
rsync -av ~/project/ <your username>@berzelius1.nsc.liu.se:~/project_backup/
```
- Copy from remote to local
```bash
rsync -av <your username>@berzelius1.nsc.liu.se:~/data/results/ ~/results/
```

### tar
- Create a `.tar` archive
```bash
tar -cf archive.tar folder_or_file
```
- Create a compressed `.tar.gz` archive
```bash
tar -czf archive.tar.gz folder_or_file
```
- Extract an archive
```bash
tar -xf archive.tar
tar -xzf archive.tar.gz
```
- Extract to a specific directory
```bash
tar -xzf archive.tar.gz -C /path/to/target-dir
```

### top
- Show processes run by the specified user
```bash
top -u <your username>
```

### watch
- Watch GPU usage live
```bash
watch -n 1 nvidia-smi
```

## LMod Commands

### module avail

- List all the available modules
```bash
module avail
```

- List modules related to a keyword
```bash
module avail <keyword>
```

### module list
- Show currently loaded modules

```bash
module list	
```

### module load
- Load a module into your environment
```bash
module load <module name>
```

### module purge
- Unload all loaded modules
```bash
module purge
```

## NSC Commands

### jobgraph

- Visualize GPU usage
```bash
jobgraph -j <jobid>
```

### jobsh

- Connect to the node where your job is running on
```bash
jobsh -j <jobid>
```

### lastjobs

- Show 10 last ended jobs in the past 30 days
```bash
lastjobs
```


### projinfo

- Show information about projects
```bash
projinfo
```

### nscquota
- Show disk usage quota
```
nscquota
```

## Mamba Commands

### mamba activate

- Activate environment	
```bash
mamba activate ENVNAME
```

### mamba create
- Create a new environment	
```bash
mamba create -n ENVNAME
```
- Create environment with Python version	
```bash
mamba create -n ENVNAME python=3.10
```

### mamba deactivate
- Deactivate a environment
```bash
mamba deactivate
```

### mamba env
- Export environment	
```bash
mamba env export -n ENVNAME>ENV.yml
```
- Import environment	
```bash
mamba env create -n ENVNAME --file ENV.yml
```

### mamba install
- Install a package 
```bash
mamba install PKGNAME
```
- Install specific version of package	
```bash
mamba install PKGNAME==3.1.4
```

### mamba list
- List installed packages	
```bash
mamba list
```

### mamba uninstall
- Uninstall package	
```bash
mamba uninstall PKGNAME
```





## SLURM Commands

### interactive

- Create an interactive session
```bash
interactive --gpus 1 -t 120 -A <your project>
```

- Create an interactive session for multiple nodes
```bash
interactive --nodes=2 --gpus-per-node=8  -t 30 -A <your project>
```

- Create an interactive session on a reservation
```bash
interactive --gpus 1 -t 120 -A <your project> --reservation=<reservation name>
```



### scancel

- Kill a job in the queue
```bash
scancel <jobid>
```

### scontrol

- Show reservations
```bash
scontrol show reservation
```

- Display detailed information about a job
```bash
scontrol show job <jobid>
```

### seff
- Report the efficiency of a completed job, based on how well it used the allocated CPU and memory resources
```bash
seff <jobid>
```

### sinfo
- Display the current status of compute nodes
```bash
sinfo
```

### squeue

- Show your running and pending jobs in the queue
```bash
squeue -u <your username>
```



