# Best Practices

Managing Conda environments effectively ensures stability, reproducibility, and ease of collaboration — especially in shared or HPC environments. Below are some recommended practices to follow.


---

## 1. Use Conda Environments for Isolation

Avoid installing packages into the base environment. Instead, create dedicated environments for each project:

```
mamba create -n my_project_env python=3.10
```

This keeps dependencies isolated, avoids conflicts, and allows for easy cleanup. Always activate the environment before installing or running anything:
```
mamba activate my_project_env
```


## 2. Use Pip Only After Conda

If a package is not available in `conda-forge` or other Conda channels, you can fall back on `pip`, but follow these rules:

- Install all Conda packages first.

- Use pip only after conda install steps are complete.

- Avoid mixing the two repeatedly — never run conda install after pip install.

- Never use `pip install --user` inside a Conda environment.

This helps avoid dependency issues and ensures consistency.


## 3. Recreate the Environment if Changes Are Needed

You should consider recreating the environment when:

- You’ve used `pip install` and want to install more packages with `conda`:
Once pip has been used, conda is unaware of the changes and may cause dependency conflicts or break the environment.

- You’re upgrading core components (e.g., Python, PyTorch, CUDA):
These changes often affect many dependencies and are cleaner to handle with a fresh environment.

- Your environment has become inconsistent or unstable:
Conflicts, broken packages, or mysterious errors can often be fixed by starting fresh.

- You’re preparing an environment to share or document:
A clean environment with a curated `environment.yml` is easier to reproduce for collaborators or on other systems.

- You want to clean out unused packages:
Over time, environments accumulate unnecessary packages. Recreating allows you to start from a minimal set.

By using this workflow, you’ll reduce technical debt and improve consistency across your systems and collaborators.


## 4. Store Conda and Pip Requirements in Text Files

For reproducibility and collaboration, always record the environment state:

- Export Conda environment:

```
mamba env export -n myenv > environment.yml
```

- Store pip packages separately (if any):

```
pip freeze > requirements.txt
```

Avoid blindly exporting pip-installed packages into the Conda YAML. Keep them in separate `requirements.txt` files if your project needs both.

## 5. Specify Package Versions for Reproducibility

Explicitly specify critical package versions (like `pytorch`, `tensorflow`, etc.) in installation commands to to ensure consistent behavior across systems and over time.

```
pytorch==2.5.0 torchvision torchaudio pytorch-cuda=12.1 -c pytorch -c nvidia
```

## 6. Review Install Plan Before Confirming

Before installing or updating packages with Conda or Mamba, carefully read the printed-out transaction plan before pressing Enter.

Conda/Mamba will print a summary of packages to be installed, updated, or downgraded. Look for unexpected changes (e.g., major version downgrades or unwanted removals).

If something looks off, abort the install (Ctrl+C) and review your dependencies — this helps prevent environment breakage and unexpected behavior.


## 7. Tips for Berzelius Users

### 7.1 Avoid Modifying `~/.bashrc` or `~/.bash_profile`

Avoid modifying your `~/.bashrc` or `~/.bash_profile` with `conda init` or similar commands. These modifications can interfere with the module system on Berzelius and break environment activation.

### 7.2 Use the Miniforge3 Module

Use the `Miniforge3` module to manage Conda environments efficiently on Berzelius:

```bash
module load Miniforge3
```

- It avoids reliance on the default `anaconda` channel, which is governed by [Anaconda's Terms of Service](https://www.anaconda.com/terms-of-service). These terms restrict usage in commercial environments and large institutions (defined as having 200 or more employees).
- Miniforge3 uses the open-source `conda-forge` channel by default, which has no usage restrictions and is widely preferred for academic and HPC contexts.
- As of July 2024, `Mambaforge` is [deprecated by the conda-forge community](https://github.com/conda-forge/miniforge). The current recommendation is to use Miniforge and install `mamba` manually into your base or project environments. This provides more flexibility and reduces maintenance overhead.


### 7.3 Clean Up Unused Environments

List all environments:

```
mamba env list
```

Remove ones you no longer need:
```
conda env remove -n old_env
```

This reduces clutter and frees up disk space, especially in shared or quota-limited systems like Berzelius.



