import os
import re
import argparse
from titlecase import titlecase

def find_lesson_dirs():
    lesson_dirs = []
    for item in os.listdir("."):
        item_path = os.path.join(item, "content")
        if os.path.isdir(item_path) and item != "landing":
            lesson_dirs.append(item)
    return lesson_dirs

def is_target_md_file(filename):
    # Match "index.md" or files starting with digits and a period
    return (
        filename == "index.md"
        or re.match(r"^\d+\.\s?.*\.md$", filename)
    )

def titlecase_headings(text):
    def replacer(match):
        hashes, space, title = match.groups()

        # Split title into parts inside and outside backticks
        parts = re.split(r"(`[^`]+`)", title)

        # Apply titlecase only to parts outside backticks
        parts = [part if part.startswith("`") and part.endswith("`") else titlecase(part) for part in parts]

        return f"{hashes}{space}{''.join(parts)}"

    return re.sub(r"^(#+)(\s+)(.+)", replacer, text, flags=re.MULTILINE)


def process_file(filepath, dry_run=False):
    with open(filepath, "r", encoding="utf-8") as f:
        original = f.read()

    updated = titlecase_headings(original)

    if original != updated:
        if dry_run:
            print(f"[DRY RUN] Would update: {filepath}")
        else:
            with open(filepath, "w", encoding="utf-8") as f:
                f.write(updated)
            print(f"✅ Updated: {filepath}")
    else:
        if dry_run:
            print(f"[DRY RUN] No change needed: {filepath}")

def main(dry_run=False):
    print("=== Finding lesson directories ===")
    lesson_dirs = find_lesson_dirs()

    for lesson in lesson_dirs:
        content_dir = os.path.join(lesson, "content")
        for filename in sorted(os.listdir(content_dir)):
            if filename.endswith(".md") and is_target_md_file(filename):
                filepath = os.path.join(content_dir, filename)
                process_file(filepath, dry_run=dry_run)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Convert Markdown headings to Title Case using titlecase library.")
    parser.add_argument("--dry-run", action="store_true", help="Preview changes without writing to files.")
    args = parser.parse_args()

    main(dry_run=args.dry_run)
